@@ -19,7 +19,6 @@ from __future__ import annotations
 
 import asyncio
 import logging
-from typing import Optional, Union
 
 import click
 
@@ -47,14 +46,13 @@ from bumble.avdtp import (
     AVDTP_DELAY_REPORTING_SERVICE_CATEGORY,
     MediaCodecCapabilities,
     MediaPacketPump,
+    find_avdtp_service_with_connection,
 )
 from bumble.avdtp import Protocol as AvdtpProtocol
-from bumble.avdtp import find_avdtp_service_with_connection
 from bumble.avrcp import Protocol as AvrcpProtocol
 from bumble.colors import color
-from bumble.core import AdvertisingData
+from bumble.core import AdvertisingData, DeviceClass, PhysicalTransport
 from bumble.core import ConnectionError as BumbleConnectionError
-from bumble.core import DeviceClass, PhysicalTransport
 from bumble.device import Connection, Device, DeviceConfiguration
 from bumble.hci import HCI_CONNECTION_ALREADY_EXISTS_ERROR, Address, HCI_Constant
 from bumble.pairing import PairingConfig
@@ -191,7 +189,7 @@ class Player:
     def __init__(
         self,
         transport: str,
-        device_config: Optional[str],
+        device_config: str | None,
         authenticate: bool,
         encrypt: bool,
     ) -> None:
@@ -199,8 +197,8 @@ class Player:
         self.device_config = device_config
         self.authenticate = authenticate
         self.encrypt = encrypt
-        self.avrcp_protocol: Optional[AvrcpProtocol] = None
-        self.done: Optional[asyncio.Event]
+        self.avrcp_protocol: AvrcpProtocol | None = None
+        self.done: asyncio.Event | None
 
     async def run(self, workload) -> None:
         self.done = asyncio.Event()
@@ -315,7 +313,7 @@ class Player:
         codec_type: int,
         vendor_id: int,
         codec_id: int,
-        packet_source: Union[SbcPacketSource, AacPacketSource, OpusPacketSource],
+        packet_source: SbcPacketSource | AacPacketSource | OpusPacketSource,
         codec_capabilities: MediaCodecCapabilities,
     ):
         # Discover all endpoints on the remote device
@@ -381,11 +379,11 @@ class Player:
             print(f">>> {color(address.to_string(False), 'yellow')}:")
             print(f"  Device Class (raw): {class_of_device:06X}")
             major_class_name = DeviceClass.major_device_class_name(major_device_class)
-            print("  Device Major Class: " f"{major_class_name}")
+            print(f"  Device Major Class: {major_class_name}")
             minor_class_name = DeviceClass.minor_device_class_name(
                 major_device_class, minor_device_class
             )
-            print("  Device Minor Class: " f"{minor_class_name}")
+            print(f"  Device Minor Class: {minor_class_name}")
             print(
                 "  Device Services: "
                 f"{', '.join(DeviceClass.service_class_labels(service_classes))}"
@@ -420,7 +418,7 @@ class Player:
     async def play(
         self,
         device: Device,
-        address: Optional[str],
+        address: str | None,
         audio_format: str,
         audio_file: str,
     ) -> None:
@@ -449,7 +447,7 @@ class Player:
                     return input_file.read(byte_count)
 
                 # Obtain the codec capabilities from the stream
-                packet_source: Union[SbcPacketSource, AacPacketSource, OpusPacketSource]
+                packet_source: SbcPacketSource | AacPacketSource | OpusPacketSource
                 vendor_id = 0
                 codec_id = 0
                 if audio_format == "sbc":
