@@ -26,7 +26,8 @@ import logging
 import pathlib
 import sys
 import wave
-from typing import TYPE_CHECKING, AsyncGenerator, BinaryIO
+from collections.abc import AsyncGenerator
+from typing import TYPE_CHECKING, BinaryIO
 
 from bumble.colors import color
 
@@ -545,5 +546,6 @@ class SoundDeviceAudioInput(ThreadedAudioInput):
         return bytes(pcm_buffer)
 
     def _close(self):
-        self._stream.stop()
-        self._stream = None
+        if self._stream:
+            self._stream.stop()
+            self._stream = None
