@@ -21,12 +21,16 @@ import asyncio
 import collections
 import dataclasses
 import logging
-import struct
-from typing import TYPE_CHECKING, Any, Awaitable, Callable, Optional, Union, cast
+from collections.abc import Awaitable, Callable
+from typing import TYPE_CHECKING, Any, TypeVar, cast, overload
 
 from bumble import drivers, hci, utils
 from bumble.colors import color
-from bumble.core import ConnectionPHY, InvalidStateError, PhysicalTransport
+from bumble.core import (
+    ConnectionPHY,
+    InvalidStateError,
+    PhysicalTransport,
+)
 from bumble.l2cap import L2CAP_PDU
 from bumble.snoop import Snooper
 from bumble.transport.common import TransportLostError
@@ -34,7 +38,6 @@ from bumble.transport.common import TransportLostError
 if TYPE_CHECKING:
     from bumble.transport.common import TransportSink, TransportSource
 
-
 # -----------------------------------------------------------------------------
 # Logging
 # -----------------------------------------------------------------------------
@@ -108,8 +111,7 @@ class DataPacketQueue(utils.EventEmitter):
 
         if self._packets:
             logger.debug(
-                f'{self._in_flight} packets in flight, '
-                f'{len(self._packets)} in queue'
+                f'{self._in_flight} packets in flight, {len(self._packets)} in queue'
             )
 
     def flush(self, connection_handle: int) -> None:
@@ -199,7 +201,7 @@ class Connection:
         self.peer_address = peer_address
         self.assembler = hci.HCI_AclDataPacketAssembler(self.on_acl_pdu)
         self.transport = transport
-        acl_packet_queue: Optional[DataPacketQueue] = (
+        acl_packet_queue: DataPacketQueue | None = (
             host.le_acl_packet_queue
             if transport == PhysicalTransport.LE
             else host.acl_packet_queue
@@ -236,26 +238,27 @@ class IsoLink:
 
 
 # -----------------------------------------------------------------------------
+_RP = TypeVar('_RP', bound=hci.HCI_ReturnParameters)
+
+
 class Host(utils.EventEmitter):
     connections: dict[int, Connection]
     cis_links: dict[int, IsoLink]
     bis_links: dict[int, IsoLink]
     sco_links: dict[int, ScoLink]
     bigs: dict[int, set[int]]
-    acl_packet_queue: Optional[DataPacketQueue] = None
-    le_acl_packet_queue: Optional[DataPacketQueue] = None
-    iso_packet_queue: Optional[DataPacketQueue] = None
-    hci_sink: Optional[TransportSink] = None
+    acl_packet_queue: DataPacketQueue | None = None
+    le_acl_packet_queue: DataPacketQueue | None = None
+    iso_packet_queue: DataPacketQueue | None = None
+    hci_sink: TransportSink | None = None
     hci_metadata: dict[str, Any]
-    long_term_key_provider: Optional[
-        Callable[[int, bytes, int], Awaitable[Optional[bytes]]]
-    ]
-    link_key_provider: Optional[Callable[[hci.Address], Awaitable[Optional[bytes]]]]
+    long_term_key_provider: Callable[[int, bytes, int], Awaitable[bytes | None]] | None
+    link_key_provider: Callable[[hci.Address], Awaitable[bytes | None]] | None
 
     def __init__(
         self,
-        controller_source: Optional[TransportSource] = None,
-        controller_sink: Optional[TransportSink] = None,
+        controller_source: TransportSource | None = None,
+        controller_sink: TransportSink | None = None,
     ) -> None:
         super().__init__()
 
@@ -266,13 +269,20 @@ class Host(utils.EventEmitter):
         self.bis_links = {}  # BIS links, by connection handle
         self.sco_links = {}  # SCO links, by connection handle
         self.bigs = {}  # BIG Handle to BIS Handles
-        self.pending_command = None
-        self.pending_response: Optional[asyncio.Future[Any]] = None
+        self.pending_command: hci.HCI_SyncCommand | hci.HCI_AsyncCommand | None = None
+        self.pending_response: (
+            asyncio.Future[
+                hci.HCI_Command_Complete_Event | hci.HCI_Command_Status_Event
+            ]
+            | None
+        ) = None
         self.number_of_supported_advertising_sets = 0
         self.maximum_advertising_data_length = 31
-        self.local_version = None
+        self.local_version: (
+            hci.HCI_Read_Local_Version_Information_ReturnParameters | None
+        ) = None
         self.local_supported_commands = 0
-        self.local_le_features = 0
+        self.local_le_features = hci.LeFeatureMask(0)  # LE features
         self.local_lmp_features = hci.LmpFeatureMask(0)  # Classic LMP features
         self.suggested_max_tx_octets = 251  # Max allowed
         self.suggested_max_tx_time = 2120  # Max allowed
@@ -280,7 +290,7 @@ class Host(utils.EventEmitter):
         self.long_term_key_provider = None
         self.link_key_provider = None
         self.pairing_io_capability_provider = None  # Classic only
-        self.snooper: Optional[Snooper] = None
+        self.snooper: Snooper | None = None
 
         # Connect to the source and sink if specified
         if controller_source:
@@ -291,9 +301,9 @@ class Host(utils.EventEmitter):
     def find_connection_by_bd_addr(
         self,
         bd_addr: hci.Address,
-        transport: Optional[int] = None,
+        transport: int | None = None,
         check_address_type: bool = False,
-    ) -> Optional[Connection]:
+    ) -> Connection | None:
         for connection in self.connections.values():
             if bytes(connection.peer_address) == bytes(bd_addr):
                 if (
@@ -314,7 +324,7 @@ class Host(utils.EventEmitter):
         self.emit('flush')
         self.command_semaphore.release()
 
-    async def reset(self, driver_factory=drivers.get_driver_for_host):
+    async def reset(self, driver_factory=drivers.get_driver_for_host) -> None:
         if self.ready:
             self.ready = False
             await self.flush()
@@ -332,57 +342,61 @@ class Host(utils.EventEmitter):
 
         # Send a reset command unless a driver has already done so.
         if reset_needed:
-            await self.send_command(hci.HCI_Reset_Command(), check_result=True)
+            await self.send_sync_command(hci.HCI_Reset_Command())
             self.ready = True
 
-        response = await self.send_command(
-            hci.HCI_Read_Local_Supported_Commands_Command(), check_result=True
+        response1 = await self.send_sync_command(
+            hci.HCI_Read_Local_Supported_Commands_Command()
         )
         self.local_supported_commands = int.from_bytes(
-            response.return_parameters.supported_commands, 'little'
+            response1.supported_commands, 'little'
         )
 
-        if self.supports_command(hci.HCI_LE_READ_LOCAL_SUPPORTED_FEATURES_COMMAND):
-            response = await self.send_command(
-                hci.HCI_LE_Read_Local_Supported_Features_Command(), check_result=True
-            )
-            self.local_le_features = struct.unpack(
-                '<Q', response.return_parameters.le_features
-            )[0]
-
         if self.supports_command(hci.HCI_READ_LOCAL_VERSION_INFORMATION_COMMAND):
-            response = await self.send_command(
-                hci.HCI_Read_Local_Version_Information_Command(), check_result=True
+            self.local_version = await self.send_sync_command(
+                hci.HCI_Read_Local_Version_Information_Command()
+            )
+
+        if self.supports_command(hci.HCI_LE_READ_ALL_LOCAL_SUPPORTED_FEATURES_COMMAND):
+            response2 = await self.send_sync_command(
+                hci.HCI_LE_Read_All_Local_Supported_Features_Command()
+            )
+            self.local_le_features = hci.LeFeatureMask(
+                int.from_bytes(response2.le_features, 'little')
+            )
+        elif self.supports_command(hci.HCI_LE_READ_LOCAL_SUPPORTED_FEATURES_COMMAND):
+            response3 = await self.send_sync_command(
+                hci.HCI_LE_Read_Local_Supported_Features_Command()
+            )
+            self.local_le_features = hci.LeFeatureMask(
+                int.from_bytes(response3.le_features, 'little')
             )
-            self.local_version = response.return_parameters
 
         if self.supports_command(hci.HCI_READ_LOCAL_EXTENDED_FEATURES_COMMAND):
             max_page_number = 0
             page_number = 0
             lmp_features = 0
             while page_number <= max_page_number:
-                response = await self.send_command(
+                response4 = await self.send_sync_command(
                     hci.HCI_Read_Local_Extended_Features_Command(
                         page_number=page_number
-                    ),
-                    check_result=True,
+                    )
                 )
                 lmp_features |= int.from_bytes(
-                    response.return_parameters.extended_lmp_features, 'little'
+                    response4.extended_lmp_features, 'little'
                 ) << (64 * page_number)
-                max_page_number = response.return_parameters.maximum_page_number
+                max_page_number = response4.maximum_page_number
                 page_number += 1
             self.local_lmp_features = hci.LmpFeatureMask(lmp_features)
-
         elif self.supports_command(hci.HCI_READ_LOCAL_SUPPORTED_FEATURES_COMMAND):
-            response = await self.send_command(
-                hci.HCI_Read_Local_Supported_Features_Command(), check_result=True
+            response5 = await self.send_sync_command(
+                hci.HCI_Read_Local_Supported_Features_Command()
             )
             self.local_lmp_features = hci.LmpFeatureMask(
-                int.from_bytes(response.return_parameters.lmp_features, 'little')
+                int.from_bytes(response5.lmp_features, 'little')
             )
 
-        await self.send_command(
+        await self.send_sync_command(
             hci.HCI_Set_Event_Mask_Command(
                 event_mask=hci.HCI_Set_Event_Mask_Command.mask(
                     [
@@ -439,7 +453,7 @@ class Host(utils.EventEmitter):
             )
         )
         if self.supports_command(hci.HCI_SET_EVENT_MASK_PAGE_2_COMMAND):
-            await self.send_command(
+            await self.send_sync_command(
                 hci.HCI_Set_Event_Mask_Page_2_Command(
                     event_mask_page_2=hci.HCI_Set_Event_Mask_Page_2_Command.mask(
                         [hci.HCI_ENCRYPTION_CHANGE_V2_EVENT]
@@ -492,29 +506,28 @@ class Host(utils.EventEmitter):
                     hci.HCI_LE_TRANSMIT_POWER_REPORTING_EVENT,
                     hci.HCI_LE_BIGINFO_ADVERTISING_REPORT_EVENT,
                     hci.HCI_LE_SUBRATE_CHANGE_EVENT,
+                    hci.HCI_LE_READ_ALL_REMOTE_FEATURES_COMPLETE_EVENT,
                     hci.HCI_LE_CS_READ_REMOTE_SUPPORTED_CAPABILITIES_COMPLETE_EVENT,
                     hci.HCI_LE_CS_PROCEDURE_ENABLE_COMPLETE_EVENT,
                     hci.HCI_LE_CS_SECURITY_ENABLE_COMPLETE_EVENT,
                     hci.HCI_LE_CS_CONFIG_COMPLETE_EVENT,
                     hci.HCI_LE_CS_SUBEVENT_RESULT_EVENT,
                     hci.HCI_LE_CS_SUBEVENT_RESULT_CONTINUE_EVENT,
+                    hci.HCI_LE_MONITORED_ADVERTISERS_REPORT_EVENT,
+                    hci.HCI_LE_FRAME_SPACE_UPDATE_COMPLETE_EVENT,
+                    hci.HCI_LE_UTP_RECEIVE_EVENT,
+                    hci.HCI_LE_CONNECTION_RATE_CHANGE_EVENT,
                 ]
             )
 
-        await self.send_command(
+        await self.send_sync_command(
             hci.HCI_LE_Set_Event_Mask_Command(le_event_mask=le_event_mask)
         )
 
         if self.supports_command(hci.HCI_READ_BUFFER_SIZE_COMMAND):
-            response = await self.send_command(
-                hci.HCI_Read_Buffer_Size_Command(), check_result=True
-            )
-            hc_acl_data_packet_length = (
-                response.return_parameters.hc_acl_data_packet_length
-            )
-            hc_total_num_acl_data_packets = (
-                response.return_parameters.hc_total_num_acl_data_packets
-            )
+            response6 = await self.send_sync_command(hci.HCI_Read_Buffer_Size_Command())
+            hc_acl_data_packet_length = response6.hc_acl_data_packet_length
+            hc_total_num_acl_data_packets = response6.hc_total_num_acl_data_packets
 
             logger.debug(
                 'HCI ACL flow control: '
@@ -533,19 +546,13 @@ class Host(utils.EventEmitter):
         iso_data_packet_length = 0
         total_num_iso_data_packets = 0
         if self.supports_command(hci.HCI_LE_READ_BUFFER_SIZE_V2_COMMAND):
-            response = await self.send_command(
-                hci.HCI_LE_Read_Buffer_Size_V2_Command(), check_result=True
-            )
-            le_acl_data_packet_length = (
-                response.return_parameters.le_acl_data_packet_length
-            )
-            total_num_le_acl_data_packets = (
-                response.return_parameters.total_num_le_acl_data_packets
-            )
-            iso_data_packet_length = response.return_parameters.iso_data_packet_length
-            total_num_iso_data_packets = (
-                response.return_parameters.total_num_iso_data_packets
+            response7 = await self.send_sync_command(
+                hci.HCI_LE_Read_Buffer_Size_V2_Command()
             )
+            le_acl_data_packet_length = response7.le_acl_data_packet_length
+            total_num_le_acl_data_packets = response7.total_num_le_acl_data_packets
+            iso_data_packet_length = response7.iso_data_packet_length
+            total_num_iso_data_packets = response7.total_num_iso_data_packets
 
             logger.debug(
                 'HCI LE flow control: '
@@ -555,15 +562,11 @@ class Host(utils.EventEmitter):
                 f'total_num_iso_data_packets={total_num_iso_data_packets}'
             )
         elif self.supports_command(hci.HCI_LE_READ_BUFFER_SIZE_COMMAND):
-            response = await self.send_command(
-                hci.HCI_LE_Read_Buffer_Size_Command(), check_result=True
-            )
-            le_acl_data_packet_length = (
-                response.return_parameters.le_acl_data_packet_length
-            )
-            total_num_le_acl_data_packets = (
-                response.return_parameters.total_num_le_acl_data_packets
+            response8 = await self.send_sync_command(
+                hci.HCI_LE_Read_Buffer_Size_Command()
             )
+            le_acl_data_packet_length = response8.le_acl_data_packet_length
+            total_num_le_acl_data_packets = response8.total_num_le_acl_data_packets
 
             logger.debug(
                 'HCI LE ACL flow control: '
@@ -594,16 +597,16 @@ class Host(utils.EventEmitter):
         ) and self.supports_command(
             hci.HCI_LE_WRITE_SUGGESTED_DEFAULT_DATA_LENGTH_COMMAND
         ):
-            response = await self.send_command(
+            response9 = await self.send_sync_command(
                 hci.HCI_LE_Read_Suggested_Default_Data_Length_Command()
             )
-            suggested_max_tx_octets = response.return_parameters.suggested_max_tx_octets
-            suggested_max_tx_time = response.return_parameters.suggested_max_tx_time
+            suggested_max_tx_octets = response9.suggested_max_tx_octets
+            suggested_max_tx_time = response9.suggested_max_tx_time
             if (
                 suggested_max_tx_octets != self.suggested_max_tx_octets
                 or suggested_max_tx_time != self.suggested_max_tx_time
             ):
-                await self.send_command(
+                await self.send_sync_command(
                     hci.HCI_LE_Write_Suggested_Default_Data_Length_Command(
                         suggested_max_tx_octets=self.suggested_max_tx_octets,
                         suggested_max_tx_time=self.suggested_max_tx_time,
@@ -613,27 +616,31 @@ class Host(utils.EventEmitter):
         if self.supports_command(
             hci.HCI_LE_READ_NUMBER_OF_SUPPORTED_ADVERTISING_SETS_COMMAND
         ):
-            response = await self.send_command(
-                hci.HCI_LE_Read_Number_Of_Supported_Advertising_Sets_Command(),
-                check_result=True,
-            )
-            self.number_of_supported_advertising_sets = (
-                response.return_parameters.num_supported_advertising_sets
-            )
+            try:
+                response10 = await self.send_sync_command(
+                    hci.HCI_LE_Read_Number_Of_Supported_Advertising_Sets_Command()
+                )
+                self.number_of_supported_advertising_sets = (
+                    response10.num_supported_advertising_sets
+                )
+            except hci.HCI_Error:
+                logger.warning('Failed to read number of supported advertising sets')
 
         if self.supports_command(
             hci.HCI_LE_READ_MAXIMUM_ADVERTISING_DATA_LENGTH_COMMAND
         ):
-            response = await self.send_command(
-                hci.HCI_LE_Read_Maximum_Advertising_Data_Length_Command(),
-                check_result=True,
-            )
-            self.maximum_advertising_data_length = (
-                response.return_parameters.max_advertising_data_length
-            )
+            try:
+                response11 = await self.send_sync_command(
+                    hci.HCI_LE_Read_Maximum_Advertising_Data_Length_Command()
+                )
+                self.maximum_advertising_data_length = (
+                    response11.max_advertising_data_length
+                )
+            except hci.HCI_Error:
+                logger.warning('Failed to read maximum advertising data length')
 
     @property
-    def controller(self) -> Optional[TransportSink]:
+    def controller(self) -> TransportSink | None:
         return self.hci_sink
 
     @controller.setter
@@ -642,7 +649,7 @@ class Host(utils.EventEmitter):
         if controller:
             self.set_packet_source(controller)
 
-    def set_packet_sink(self, sink: Optional[TransportSink]) -> None:
+    def set_packet_sink(self, sink: TransportSink | None) -> None:
         self.hci_sink = sink
 
     def set_packet_source(self, source: TransportSource) -> None:
@@ -656,58 +663,177 @@ class Host(utils.EventEmitter):
         if self.hci_sink:
             self.hci_sink.on_packet(bytes(packet))
 
-    async def send_command(
-        self, command, check_result=False, response_timeout: Optional[int] = None
-    ):
+    async def _send_command(
+        self,
+        command: hci.HCI_SyncCommand | hci.HCI_AsyncCommand,
+        response_timeout: float | None = None,
+    ) -> hci.HCI_Command_Complete_Event | hci.HCI_Command_Status_Event:
         # Wait until we can send (only one pending command at a time)
-        async with self.command_semaphore:
-            assert self.pending_command is None
-            assert self.pending_response is None
+        await self.command_semaphore.acquire()
 
-            # Create a future value to hold the eventual response
-            self.pending_response = asyncio.get_running_loop().create_future()
-            self.pending_command = command
+        # Create a future value to hold the eventual response
+        assert self.pending_command is None
+        assert self.pending_response is None
+        self.pending_response = asyncio.get_running_loop().create_future()
+        self.pending_command = command
 
-            try:
-                self.send_hci_packet(command)
-                await asyncio.wait_for(self.pending_response, timeout=response_timeout)
-                response = self.pending_response.result()
+        response: (
+            hci.HCI_Command_Complete_Event | hci.HCI_Command_Status_Event | None
+        ) = None
+        try:
+            self.send_hci_packet(command)
+            response = await asyncio.wait_for(
+                self.pending_response, timeout=response_timeout
+            )
+            return response
+        except Exception:
+            logger.exception(color("!!! Exception while sending command:", "red"))
+            raise
+        finally:
+            self.pending_command = None
+            self.pending_response = None
+            if (
+                response is not None
+                and response.num_hci_command_packets
+                and self.command_semaphore.locked()
+            ):
+                self.command_semaphore.release()
 
-                # Check the return parameters if required
-                if check_result:
-                    if isinstance(response, hci.HCI_Command_Status_Event):
-                        status = response.status  # type: ignore[attr-defined]
-                    elif isinstance(response.return_parameters, int):
-                        status = response.return_parameters
-                    elif isinstance(response.return_parameters, bytes):
-                        # return parameters first field is a one byte status code
-                        status = response.return_parameters[0]
-                    else:
-                        status = response.return_parameters.status
+    @overload
+    async def send_command(
+        self,
+        command: hci.HCI_SyncCommand[_RP],
+        check_result: bool = False,
+        response_timeout: float | None = None,
+    ) -> hci.HCI_Command_Complete_Event[_RP]: ...
 
-                    if status != hci.HCI_SUCCESS:
-                        logger.warning(
-                            f'{command.name} failed '
-                            f'({hci.HCI_Constant.error_name(status)})'
-                        )
-                        raise hci.HCI_Error(status)
+    @overload
+    async def send_command(
+        self,
+        command: hci.HCI_AsyncCommand,
+        check_result: bool = False,
+        response_timeout: float | None = None,
+    ) -> hci.HCI_Command_Status_Event: ...
 
-                return response
-            except Exception:
-                logger.exception(color("!!! Exception while sending command:", "red"))
-                raise
-            finally:
-                self.pending_command = None
-                self.pending_response = None
+    async def send_command(
+        self,
+        command: hci.HCI_SyncCommand[_RP] | hci.HCI_AsyncCommand,
+        check_result: bool = False,
+        response_timeout: float | None = None,
+    ) -> hci.HCI_Command_Complete_Event[_RP] | hci.HCI_Command_Status_Event:
+        response = await self._send_command(command, response_timeout)
 
-    # Use this method to send a command from a task
-    def send_command_sync(self, command: hci.HCI_Command) -> None:
-        async def send_command(command: hci.HCI_Command) -> None:
-            await self.send_command(command)
+        # Check the return parameters if required
+        if check_result:
+            if isinstance(response, hci.HCI_Command_Status_Event):
+                status = response.status  # type: ignore[attr-defined]
+            elif isinstance(response.return_parameters, int):
+                status = response.return_parameters
+            elif isinstance(response.return_parameters, bytes):
+                # return parameters first field is a one byte status code
+                status = response.return_parameters[0]
+            elif isinstance(
+                response.return_parameters, hci.HCI_GenericReturnParameters
+            ):
+                # FIXME: temporary workaround
+                # NO STATUS
+                status = hci.HCI_SUCCESS
+            else:
+                status = response.return_parameters.status
 
-        asyncio.create_task(send_command(command))
+            if status != hci.HCI_SUCCESS:
+                logger.warning(
+                    f'{command.name} failed ' f'({hci.HCI_Constant.error_name(status)})'
+                )
+                raise hci.HCI_Error(status)
 
-    def send_l2cap_pdu(self, connection_handle: int, cid: int, pdu: bytes) -> None:
+        return response
+
+    async def send_sync_command(
+        self, command: hci.HCI_SyncCommand[_RP], response_timeout: float | None = None
+    ) -> _RP:
+        response = await self.send_sync_command_raw(command, response_timeout)
+        return_parameters = response.return_parameters
+
+        # Check the return parameters's status
+        if isinstance(return_parameters, hci.HCI_StatusReturnParameters):
+            status = return_parameters.status
+        elif isinstance(return_parameters, hci.HCI_GenericReturnParameters):
+            # if the payload has at least one byte, assume the first byte is the status
+            if not return_parameters.data:
+                raise RuntimeError('no status byte in return parameters')
+            status = hci.HCI_ErrorCode(return_parameters.data[0])
+        else:
+            raise RuntimeError(
+                f'unexpected return parameters type ({type(return_parameters)})'
+            )
+        if status != hci.HCI_ErrorCode.SUCCESS:
+            logger.warning(
+                f'{command.name} failed ' f'({hci.HCI_Constant.error_name(status)})'
+            )
+            raise hci.HCI_Error(status)
+
+        return return_parameters
+
+    async def send_sync_command_raw(
+        self,
+        command: hci.HCI_SyncCommand[_RP],
+        response_timeout: float | None = None,
+    ) -> hci.HCI_Command_Complete_Event[_RP]:
+        response = await self._send_command(command, response_timeout)
+
+        # For unknown HCI commands, some controllers return Command Status instead of
+        # Command Complete.
+        if (
+            isinstance(response, hci.HCI_Command_Status_Event)
+            and response.status == hci.HCI_ErrorCode.UNKNOWN_HCI_COMMAND_ERROR
+        ):
+            return hci.HCI_Command_Complete_Event(
+                num_hci_command_packets=response.num_hci_command_packets,
+                command_opcode=command.op_code,
+                return_parameters=hci.HCI_StatusReturnParameters(
+                    status=hci.HCI_ErrorCode(response.status)
+                ),  # type: ignore
+            )
+
+        # Check that the response is of the expected type
+        assert isinstance(response, hci.HCI_Command_Complete_Event)
+
+        return response
+
+    async def send_async_command(
+        self,
+        command: hci.HCI_AsyncCommand,
+        check_status: bool = True,
+        response_timeout: float | None = None,
+    ) -> hci.HCI_ErrorCode:
+        response = await self._send_command(command, response_timeout)
+
+        # For unknown HCI commands, some controllers return Command Complete instead of
+        # Command Status.
+        if isinstance(response, hci.HCI_Command_Complete_Event):
+            # Assume the first byte of the return parameters is the status
+            if (
+                status := hci.HCI_ErrorCode(response.parameters[3])
+            ) != hci.HCI_ErrorCode.UNKNOWN_HCI_COMMAND_ERROR:
+                logger.warning(f'unexpected return paramerers status {status}')
+        else:
+            assert isinstance(response, hci.HCI_Command_Status_Event)
+            status = hci.HCI_ErrorCode(response.status)
+
+        # Check the status if required
+        if check_status:
+            if status != hci.HCI_CommandStatus.PENDING:
+                logger.warning(f'{command.name} failed ' f'({status.name})')
+                raise hci.HCI_Error(status)
+
+        return status
+
+    @utils.deprecated("Use utils.AsyncRunner.spawn() instead.")
+    def send_command_sync(self, command: hci.HCI_AsyncCommand) -> None:
+        utils.AsyncRunner.spawn(self.send_async_command(command))
+
+    def send_acl_sdu(self, connection_handle: int, sdu: bytes) -> None:
         if not (connection := self.connections.get(connection_handle)):
             logger.warning(f'connection 0x{connection_handle:04X} not found')
             return
@@ -718,27 +844,36 @@ class Host(utils.EventEmitter):
             )
             return
 
-        # Create a PDU
-        l2cap_pdu = bytes(L2CAP_PDU(cid, pdu))
-
         # Send the data to the controller via ACL packets
-        bytes_remaining = len(l2cap_pdu)
-        offset = 0
-        pb_flag = 0
-        while bytes_remaining:
-            data_total_length = min(bytes_remaining, packet_queue.max_packet_size)
+        max_packet_size = packet_queue.max_packet_size
+        for offset in range(0, len(sdu), max_packet_size):
+            pdu = sdu[offset : offset + max_packet_size]
             acl_packet = hci.HCI_AclDataPacket(
                 connection_handle=connection_handle,
-                pb_flag=pb_flag,
+                pb_flag=1 if offset > 0 else 0,
                 bc_flag=0,
-                data_total_length=data_total_length,
-                data=l2cap_pdu[offset : offset + data_total_length],
+                data_total_length=len(pdu),
+                data=pdu,
+            )
+            logger.debug(
+                '>>> ACL packet enqueue: (handle=0x%04X) %s',
+                connection_handle,
+                pdu.hex(),
             )
-            logger.debug(f'>>> ACL packet enqueue: (CID={cid}) {acl_packet}')
             packet_queue.enqueue(acl_packet, connection_handle)
-            pb_flag = 1
-            offset += data_total_length
-            bytes_remaining -= data_total_length
+
+    def send_sco_sdu(self, connection_handle: int, sdu: bytes) -> None:
+        self.send_hci_packet(
+            hci.HCI_SynchronousDataPacket(
+                connection_handle=connection_handle,
+                packet_status=0,
+                data_total_length=len(sdu),
+                data=sdu,
+            )
+        )
+
+    def send_l2cap_pdu(self, connection_handle: int, cid: int, pdu: bytes) -> None:
+        self.send_acl_sdu(connection_handle, bytes(L2CAP_PDU(cid, pdu)))
 
     def get_data_packet_queue(self, connection_handle: int) -> DataPacketQueue | None:
         if connection := self.connections.get(connection_handle):
@@ -821,16 +956,18 @@ class Host(utils.EventEmitter):
             if self.local_supported_commands & mask
         )
 
-    def supports_le_features(self, feature: hci.LeFeatureMask) -> bool:
-        return (self.local_le_features & feature) == feature
+    def supports_le_features(self, features: hci.LeFeatureMask) -> bool:
+        return (self.local_le_features & features) == features
 
-    def supports_lmp_features(self, feature: hci.LmpFeatureMask) -> bool:
-        return self.local_lmp_features & (feature) == feature
+    def supports_lmp_features(self, features: hci.LmpFeatureMask) -> bool:
+        return self.local_lmp_features & (features) == features
 
     @property
-    def supported_le_features(self):
+    def supported_le_features(self) -> list[hci.LeFeature]:
         return [
-            feature for feature in range(64) if self.local_le_features & (1 << feature)
+            feature
+            for feature in hci.LeFeature
+            if self.local_le_features & (1 << feature)
         ]
 
     # Packet Sink protocol (packets coming from the controller via HCI)
@@ -903,7 +1040,7 @@ class Host(utils.EventEmitter):
         self.emit('l2cap_pdu', connection.handle, cid, pdu)
 
     def on_command_processed(
-        self, event: Union[hci.HCI_Command_Complete_Event, hci.HCI_Command_Status_Event]
+        self, event: hci.HCI_Command_Complete_Event | hci.HCI_Command_Status_Event
     ):
         if self.pending_response:
             # Check that it is what we were expecting
@@ -919,6 +1056,8 @@ class Host(utils.EventEmitter):
             self.pending_response.set_result(event)
         else:
             logger.warning('!!! no pending response future to set')
+            if event.num_hci_command_packets and self.command_semaphore.locked():
+                self.command_semaphore.release()
 
     ############################################################
     # HCI handlers
@@ -930,7 +1069,13 @@ class Host(utils.EventEmitter):
         if event.command_opcode == 0:
             # This is used just for the Num_HCI_Command_Packets field, not related to
             # an actual command
-            logger.debug('no-command event')
+            logger.debug('no-command event for flow control')
+
+            # Release the command semaphore if needed
+            if event.num_hci_command_packets and self.command_semaphore.locked():
+                logger.debug('command complete event releasing semaphore')
+                self.command_semaphore.release()
+
             return
 
         return self.on_command_processed(event)
@@ -966,11 +1111,11 @@ class Host(utils.EventEmitter):
 
     def on_hci_le_connection_complete_event(
         self,
-        event: Union[
-            hci.HCI_LE_Connection_Complete_Event,
-            hci.HCI_LE_Enhanced_Connection_Complete_Event,
-            hci.HCI_LE_Enhanced_Connection_Complete_V2_Event,
-        ],
+        event: (
+            hci.HCI_LE_Connection_Complete_Event
+            | hci.HCI_LE_Enhanced_Connection_Complete_Event
+            | hci.HCI_LE_Enhanced_Connection_Complete_V2_Event
+        ),
     ):
         # Check if this is a cancellation
         if event.status == hci.HCI_SUCCESS:
@@ -1015,10 +1160,10 @@ class Host(utils.EventEmitter):
 
     def on_hci_le_enhanced_connection_complete_event(
         self,
-        event: Union[
-            hci.HCI_LE_Enhanced_Connection_Complete_Event,
-            hci.HCI_LE_Enhanced_Connection_Complete_V2_Event,
-        ],
+        event: (
+            hci.HCI_LE_Enhanced_Connection_Complete_Event
+            | hci.HCI_LE_Enhanced_Connection_Complete_V2_Event
+        ),
     ):
         # Just use the same implementation as for the non-enhanced event for now
         self.on_hci_le_connection_complete_event(event)
@@ -1111,7 +1256,7 @@ class Host(utils.EventEmitter):
         self, event: hci.HCI_LE_Connection_Update_Complete_Event
     ):
         if (connection := self.connections.get(event.connection_handle)) is None:
-            logger.warning('!!! CONNECTION PARAMETERS UPDATE COMPLETE: unknown handle')
+            logger.warning('!!! CONNECTION UPDATE COMPLETE: unknown handle')
             return
 
         # Notify the client
@@ -1128,6 +1273,29 @@ class Host(utils.EventEmitter):
                 'connection_parameters_update_failure', connection.handle, event.status
             )
 
+    def on_hci_le_connection_rate_change_event(
+        self, event: hci.HCI_LE_Connection_Rate_Change_Event
+    ):
+        if (connection := self.connections.get(event.connection_handle)) is None:
+            logger.warning('!!! CONNECTION RATE CHANGE: unknown handle')
+            return
+
+        # Notify the client
+        if event.status == hci.HCI_SUCCESS:
+            self.emit(
+                'le_connection_rate_change',
+                connection.handle,
+                event.connection_interval,
+                event.subrate_factor,
+                event.peripheral_latency,
+                event.continuation_number,
+                event.supervision_timeout,
+            )
+        else:
+            self.emit(
+                'le_connection_rate_change_failure', connection.handle, event.status
+            )
+
     def on_hci_le_phy_update_complete_event(
         self, event: hci.HCI_LE_PHY_Update_Complete_Event
     ):
@@ -1343,15 +1511,17 @@ class Host(utils.EventEmitter):
 
         # For now, just accept everything
         # TODO: delegate the decision
-        self.send_command_sync(
-            hci.HCI_LE_Remote_Connection_Parameter_Request_Reply_Command(
-                connection_handle=event.connection_handle,
-                interval_min=event.interval_min,
-                interval_max=event.interval_max,
-                max_latency=event.max_latency,
-                timeout=event.timeout,
-                min_ce_length=0,
-                max_ce_length=0,
+        utils.AsyncRunner.spawn(
+            self.send_sync_command(
+                hci.HCI_LE_Remote_Connection_Parameter_Request_Reply_Command(
+                    connection_handle=event.connection_handle,
+                    interval_min=event.interval_min,
+                    interval_max=event.interval_max,
+                    max_latency=event.max_latency,
+                    timeout=event.timeout,
+                    min_ce_length=0,
+                    max_ce_length=0,
+                )
             )
         )
 
@@ -1387,9 +1557,9 @@ class Host(utils.EventEmitter):
                     connection_handle=event.connection_handle
                 )
 
-            await self.send_command(response)
+            await self.send_sync_command(response)
 
-        asyncio.create_task(send_long_term_key())
+        utils.AsyncRunner.spawn(send_long_term_key())
 
     def on_hci_synchronous_connection_complete_event(
         self, event: hci.HCI_Synchronous_Connection_Complete_Event
@@ -1397,8 +1567,7 @@ class Host(utils.EventEmitter):
         if event.status == hci.HCI_SUCCESS:
             # Create/update the connection
             logger.debug(
-                f'### SCO CONNECTION: [0x{event.connection_handle:04X}] '
-                f'{event.bd_addr}'
+                f'### SCO CONNECTION: [0x{event.connection_handle:04X}] {event.bd_addr}'
             )
 
             self.sco_links[event.connection_handle] = ScoLink(
@@ -1450,7 +1619,7 @@ class Host(utils.EventEmitter):
     def on_hci_le_data_length_change_event(
         self, event: hci.HCI_LE_Data_Length_Change_Event
     ):
-        if (connection := self.connections.get(event.connection_handle)) is None:
+        if event.connection_handle not in self.connections:
             logger.warning('!!! DATA LENGTH CHANGE: unknown handle')
             return
 
@@ -1589,9 +1758,9 @@ class Host(utils.EventEmitter):
                     bd_addr=event.bd_addr
                 )
 
-            await self.send_command(response)
+            await self.send_sync_command(response)
 
-        asyncio.create_task(send_link_key())
+        utils.AsyncRunner.spawn(send_link_key())
 
     def on_hci_io_capability_request_event(
         self, event: hci.HCI_IO_Capability_Request_Event
@@ -1686,12 +1855,13 @@ class Host(utils.EventEmitter):
             self.emit(
                 'le_remote_features_failure', event.connection_handle, event.status
             )
-        else:
-            self.emit(
-                'le_remote_features',
-                event.connection_handle,
-                int.from_bytes(event.le_features, 'little'),
-            )
+            return
+
+        self.emit(
+            'le_remote_features',
+            event.connection_handle,
+            hci.LeFeatureMask(int.from_bytes(event.le_features, 'little')),
+        )
 
     def on_hci_le_cs_read_remote_supported_capabilities_complete_event(
         self, event: hci.HCI_LE_CS_Read_Remote_Supported_Capabilities_Complete_Event
@@ -1724,6 +1894,12 @@ class Host(utils.EventEmitter):
         self.emit('cs_subevent_result_continue', event)
 
     def on_hci_le_subrate_change_event(self, event: hci.HCI_LE_Subrate_Change_Event):
+        if event.status != hci.HCI_SUCCESS:
+            self.emit(
+                'le_subrate_change_failure', event.connection_handle, event.status
+            )
+            return
+
         self.emit(
             'le_subrate_change',
             event.connection_handle,
