@@ -23,9 +23,10 @@ from unittest import mock
 
 import pytest
 
-from bumble import device, gatt, hci, utils
+from bumble import gatt, hci, utils
 from bumble.core import PhysicalTransport
 from bumble.device import (
+    Advertisement,
     AdvertisingEventProperties,
     AdvertisingParameters,
     CigParameters,
@@ -41,7 +42,6 @@ from bumble.hci import (
     HCI_CREATE_CONNECTION_COMMAND,
     HCI_SUCCESS,
     Address,
-    HCI_Command_Complete_Event,
     HCI_Command_Status_Event,
     HCI_Connection_Complete_Event,
     HCI_Connection_Request_Event,
@@ -146,17 +146,17 @@ async def test_device_connect_parallel():
             )
         )
 
-        assert (yield) == None
+        assert (yield) is None
 
     def d1_flow():
         packet = HCI_Packet.from_bytes((yield))
         assert packet.name == 'HCI_ACCEPT_CONNECTION_REQUEST_COMMAND'
 
         d1.host.on_hci_packet(
-            HCI_Command_Complete_Event(
+            HCI_Command_Status_Event(
+                status=HCI_COMMAND_STATUS_PENDING,
                 num_hci_command_packets=1,
                 command_opcode=HCI_ACCEPT_CONNECTION_REQUEST_COMMAND,
-                return_parameters=b"\x00",
             )
         )
 
@@ -180,17 +180,17 @@ async def test_device_connect_parallel():
             )
         )
 
-        assert (yield) == None
+        assert (yield) is None
 
     def d2_flow():
         packet = HCI_Packet.from_bytes((yield))
         assert packet.name == 'HCI_ACCEPT_CONNECTION_REQUEST_COMMAND'
 
         d2.host.on_hci_packet(
-            HCI_Command_Complete_Event(
+            HCI_Command_Status_Event(
+                status=HCI_COMMAND_STATUS_PENDING,
                 num_hci_command_packets=1,
                 command_opcode=HCI_ACCEPT_CONNECTION_REQUEST_COMMAND,
-                return_parameters=b"\x00",
             )
         )
 
@@ -214,7 +214,7 @@ async def test_device_connect_parallel():
             )
         )
 
-        assert (yield) == None
+        assert (yield) is None
 
     d0.host.set_packet_sink(Sink(d0_flow()))
     d1.host.set_packet_sink(Sink(d1_flow()))
@@ -239,10 +239,10 @@ async def test_device_connect_parallel():
         ]
     )
 
-    assert type(c01) == Connection
-    assert type(c02) == Connection
-    assert type(a10) == Connection
-    assert type(a20) == Connection
+    assert isinstance(c01, Connection)
+    assert isinstance(c02, Connection)
+    assert isinstance(a10, Connection)
+    assert isinstance(a20, Connection)
 
     assert c01.handle == a10.handle and c01.handle == 0x100
     assert c02.handle == a20.handle and c02.handle == 0x101
@@ -284,52 +284,51 @@ async def test_legacy_advertising():
 @pytest.mark.asyncio
 async def test_legacy_advertising_disconnection(auto_restart):
     devices = TwoDevices()
-    device = devices[0]
-    devices.controllers[0].le_features = bytes.fromhex('ffffffffffffffff')
-    await device.power_on()
-    peer_address = Address('F0:F1:F2:F3:F4:F5')
-    await device.start_advertising(auto_restart=auto_restart)
-    device.on_le_connection(
-        0x0001,
-        peer_address,
-        None,
-        None,
-        Role.PERIPHERAL,
-        0,
-        0,
-        0,
+    for controller in devices.controllers:
+        controller.le_features |= hci.LeFeatureMask.LE_EXTENDED_ADVERTISING
+    for dev in devices:
+        await dev.power_on()
+    await devices[0].start_advertising(
+        auto_restart=auto_restart, advertising_interval_min=1.0
     )
+    connection = await devices[1].connect(devices[0].random_address)
 
-    device.on_advertising_set_termination(
-        HCI_SUCCESS, device.legacy_advertising_set.advertising_handle, 0x0001, 0
-    )
+    await connection.disconnect()
 
-    device.on_disconnection(0x0001, 0)
     await async_barrier()
     await async_barrier()
 
     if auto_restart:
-        assert device.legacy_advertising_set
+        assert devices[0].legacy_advertising_set
         started = asyncio.Event()
-        if not device.is_advertising:
-            device.legacy_advertising_set.once('start', started.set)
+        if not devices[0].is_advertising:
+            devices[0].legacy_advertising_set.once('start', started.set)
             await asyncio.wait_for(started.wait(), _TIMEOUT)
-        assert device.is_advertising
+        assert devices[0].is_advertising
     else:
-        assert not device.is_advertising
+        assert not devices[0].is_advertising
 
 
 # -----------------------------------------------------------------------------
 @pytest.mark.asyncio
-async def test_extended_advertising():
-    device = TwoDevices()[0]
-    await device.power_on()
+async def test_advertising_and_scanning():
+    devices = TwoDevices()
+    for dev in devices:
+        await dev.power_on()
+
+    # Start scanning
+    advertisements = asyncio.Queue[Advertisement]()
+    devices[1].on(devices[1].EVENT_ADVERTISEMENT, advertisements.put_nowait)
+    await devices[1].start_scanning()
 
     # Start advertising
-    advertising_set = await device.create_advertising_set()
-    assert device.extended_advertising_sets
+    advertising_set = await devices[0].create_advertising_set(advertising_data=b'123')
+    assert devices[0].extended_advertising_sets
     assert advertising_set.enabled
 
+    advertisement = await asyncio.wait_for(advertisements.get(), _TIMEOUT)
+    assert advertisement.data_bytes == b'123'
+
     # Stop advertising
     await advertising_set.stop()
     assert not advertising_set.enabled
@@ -342,33 +341,33 @@ async def test_extended_advertising():
 )
 @pytest.mark.asyncio
 async def test_extended_advertising_connection(own_address_type):
-    device = TwoDevices()[0]
-    await device.power_on()
-    peer_address = Address('F0:F1:F2:F3:F4:F5')
-    advertising_set = await device.create_advertising_set(
-        advertising_parameters=AdvertisingParameters(own_address_type=own_address_type)
+    devices = TwoDevices()
+    for dev in devices:
+        await dev.power_on()
+    advertising_set = await devices[0].create_advertising_set(
+        advertising_parameters=AdvertisingParameters(
+            own_address_type=own_address_type, primary_advertising_interval_min=1.0
+        )
     )
-    device.on_le_connection(
-        0x0001,
-        peer_address,
-        None,
-        None,
-        Role.PERIPHERAL,
-        0,
-        0,
-        0,
-    )
-    device.on_advertising_set_termination(
-        HCI_SUCCESS,
-        advertising_set.advertising_handle,
-        0x0001,
-        0,
+    await asyncio.wait_for(
+        devices[1].connect(advertising_set.random_address or devices[0].public_address),
+        _TIMEOUT,
     )
+    await async_barrier()
+
+    # Advertising set should be terminated after connected.
+    assert not advertising_set.enabled
 
     if own_address_type == OwnAddressType.PUBLIC:
-        assert device.lookup_connection(0x0001).self_address == device.public_address
+        assert (
+            devices[0].lookup_connection(0x0001).self_address
+            == devices[0].public_address
+        )
     else:
-        assert device.lookup_connection(0x0001).self_address == device.random_address
+        assert (
+            devices[0].lookup_connection(0x0001).self_address
+            == devices[0].random_address
+        )
 
     await async_barrier()
 
@@ -382,7 +381,7 @@ async def test_extended_advertising_connection(own_address_type):
 async def test_extended_advertising_connection_out_of_order(own_address_type):
     devices = TwoDevices()
     device = devices[0]
-    devices.controllers[0].le_features = bytes.fromhex('ffffffffffffffff')
+    devices.controllers[0].le_features |= hci.LeFeatureMask.LE_EXTENDED_ADVERTISING
     await device.power_on()
     advertising_set = await device.create_advertising_set(
         advertising_parameters=AdvertisingParameters(own_address_type=own_address_type)
@@ -620,7 +619,9 @@ async def test_le_request_subrate():
     def on_le_subrate_change():
         q.put_nowait(lambda: None)
 
-    devices.connections[0].on(Connection.EVENT_LE_SUBRATE_CHANGE, on_le_subrate_change)
+    devices.connections[0].on(
+        Connection.EVENT_CONNECTION_PARAMETERS_UPDATE, on_le_subrate_change
+    )
 
     await devices[0].send_command(
         hci.HCI_LE_Subrate_Request_Command(
